% Replication file for
% "Who Should Be Treated?
%  Welfare Maximization Methods for Treatment Choice"
% by Toru Kitagawa and Aleksey Tetenov
% 2017-11-27
% Plot OLS plug-in treatment rules
% linear in (education, prior earnings)
% and compute IPW estimates of the welfare gain

clear all
color = true;
if (color)
    patch1_edge  = [0.6 0.4 0.1];
    patch1_color = [0.93 0.79 0.57];
    patch2_edge  = [0.3 0.4 0.3];
    patch2_color = [0.6 0.7 0.6];
    Face2Alpha    = 0.7;
    savefilename ='Figure_OLS_color';
else
    patch1_edge  = [0.2 0.2 0.2];
    patch1_color = [0.5 0.5 0.5];
    patch2_edge  = [0.5 0.5 0.5];
    patch2_color = [0.8 0.8 0.8];
    Face2Alpha    = 0.7;
    savefilename = 'Figure_OLS';
end

% DATA INPUT AND PREPARATION
load('../jtpa_kt.mat')

D = logical(D);
tcost = 0;    
Y = earnings-tcost.*D;
meanY = mean(Y);
Y = Y - meanY; % demean outcomes

tcost = 774;
Y2 = earnings-tcost.*D;
meanY2 = mean(Y2);
Y2 = Y2 - meanY2;
n = length(Y);   % sample size
ps = 2/3; % propensity score
% create regressor matrix X from the data
covariates  = [edu prevearn];

X = [ones(n,1) covariates];

g = Y.*(D/ps - (1-D)/(1-ps)); % elements of W(G) - W(G_0)
g2 = Y2.*(D/ps - (1-D)/(1-ps));
% number of variables (including the constant)
k = size(X,2);

% OLS
XT0 = X(~D,:); YT0 = Y(~D); b_ols0 = inv(XT0'*XT0)*(XT0')*YT0;
XT1 = X( D,:); YT1 = Y( D); b_ols1 = inv(XT1'*XT1)*(XT1')*YT1;
beta1 = b_ols1-b_ols0;
beta2 = beta1; beta2(1) = beta2(1)-tcost;
treat1 = (X*beta1>0);
treat2 = (X*beta2>0);

% Prepare the density plot of the covariates
[Xr, Ind] = sortrows(X);
Xu = unique(Xr,'rows');
nu  = size(Xu,1); % number of unique covariate vectors
nw = zeros(nu,1);
jj = 1;
for j = 1:size(Xr,1)
    if ~(sum(Xr(j,:)~=Xu(jj,:)))
        nw(jj) = nw(jj) + 1;
    else
        jj = jj+1;
        nw(jj) = nw(jj) + 1;
    end
end
edu_u = Xu(:,2); 
prevearn_u = Xu(:,3);


h = figure('Color','white');

line_edu = [min(edu)-0.5:0.01:max(edu)+0.5]';
% Calculate the earnings cutoff at different edu levels
% No cost - using treatment rule with parameters beta1
line_earn1 = -(beta1(1) + line_edu*beta1(2))./beta1(3);

if (beta1(3)>0) % treatment rule is increasing in earnings
    select1 = (line_earn1 <= 25000);
    patch1_X = [line_edu(select1); flipud(line_edu(select1))];
    patch1_Y = [max(line_earn1(select1),-1000); 25000*ones(sum(select1),1)];
else            % treatment rule is decreasing in earnings
    select1 = (line_earn1 >= -1000);
    patch1_X = [line_edu(select1); flipud(line_edu(select1))];
    patch1_Y = [min(line_earn1(select1),25000); -1000*ones(sum(select1),1)];
end
patch(patch1_X,patch1_Y,patch1_color,'LineStyle','-','FaceAlpha',Face2Alpha,'EdgeColor',patch1_edge);
hold on

% prepare patch 2
% Calculate the earnings cutoff at different edu levels
% With cost - using treatment rule with parameters beta2
line_earn2 = -(beta2(1) + line_edu*beta2(2))./beta2(3);

if (beta2(3)>0) % treatment rule is increasing in earnings
    select2 = (line_earn2 <= 25000);
    patch2_X = [line_edu(select2); flipud(line_edu(select2))];
    patch2_Y = [max(line_earn2(select2),-1000); 25000*ones(sum(select2),1)];
else            % treatment rule is decreasing in earnings
    select2 = (line_earn2 >= -1000);
    patch2_X = [line_edu(select2); flipud(line_edu(select2))];
    patch2_Y = [min(line_earn2(select2),25000); -1000*ones(sum(select2),1)];
end
patch(patch2_X,patch2_Y,patch2_color,'LineStyle','-','FaceAlpha',Face2Alpha,'EdgeColor',patch2_edge);

% overlay the density plot of the covariates
scatter(edu_u,prevearn_u,nw,'MarkerEdgeColor','none','MarkerFaceColor','black');

% plot labeling
axis([min(edu)-1 max(edu)+1 -2000 25000]);
xlabel('Years of education');
ylabel('Pre-program annual earnings');
ax = gca;
ax.YTick = [0,10000,20000];
ax.YTickLabel = {'$0','$10K','$20K'};

legend('OLS plug-in rule, no cost','OLS plug-in rule, $774 cost per assignee','Population density','Location','northwest');

saveas(h,savefilename,'epsc');
saveas(h,savefilename,'png');
saveas(h,savefilename,'fig');

% Output for the comparison table
diary('jtpa_OLS.log');
fprintf('Treatment cost:\n%d\n',0);
fprintf('Proportion treated:\n%.2f\n',mean(treat1));
fprintf('Est. welfare gain:\n%.0f\n',mean(g.*treat1));
fprintf('Treatment cost:\n%d\n',tcost);
fprintf('Proportion treated:\n%.2f\n',mean(treat2));
fprintf('Est. welfare gain:\n%.0f\n',mean(g2.*treat2));
diary off
