% Replication file for
% "Who Should Be Treated?
%  Welfare Maximization Methods for Treatment Choice"
% by Toru Kitagawa and Aleksey Tetenov
% 2017-11-27
% Compute quadrant EWM treatment rules in (education, prior earnings)
% Runtime (with 399 bootstrap reps):
% ~20 hours on a 4-core 4GHz Intel i7-6700K processor

clear all
rng(0); % Starting point for the RNG
bsimax = 399; % number of bootstrap reps, 0 = no bootstrap
cost = false; % switch: take treatment costs into account or not
if (cost)
    tcost = 774;
    diary('jtpa_quad_cost.log');
    filename_coefs = 'jtpa_quad_cost_coefs.mat';
    filename_results = 'jtpa_quad_cost_results.mat';
else
    tcost = 0;
    diary('jtpa_quad.log');
    filename_coefs = 'jtpa_quad_coefs.mat';
    filename_results = 'jtpa_quad_results.mat';
end

% Setup CPLEX optimization options
opt = cplexoptimset('cplex');
opt.parallel = 1;
opt.threads = 4;
opt.simplex.tolerances.feasibility = 1e-08;
opt.mip.tolerances.integrality = 1e-08;
opt.mip.strategy.variableselect = 3;
opt.mip.strategy.nodeselect = 2;
opt.mip.strategy.lbheur = 1;
opt.mip.limits.cutpasses = -1;
opt.display = 'off';

% DATA INPUT AND PREPARATION
load('../jtpa_kt.mat')

Y = earnings-tcost.*D;
meanY = mean(Y);
Y = Y - meanY; % demean outcomes
Yscale = max(abs(Y));
Y = Y./Yscale; % rescale demeaned outcomes to [-1,1]
n = length(Y); % sample size
ps = 2/3; % propensity score
% create regressor matrix X from the data
covariates  = [edu prevearn];

% Rescale covariates to fit in [-1,1]
Xscale = ones(n,1)*max(abs(covariates));
X = [ones(n,1) covariates./Xscale];

g = Y.*(D/ps - (1-D)/(1-ps)); % elements of W(G) - W(G_0)
% number of variables (including the constant)
k = 2;

vhats_p = zeros(bsimax,1); 
vhats_n = zeros(bsimax,1); 
% first BS permutation
bsperm = [1:n]';
gn = g(bsperm,:);
Xn = X(bsperm,:);
tic
for bsi = 0:bsimax
    % DRAW BOOTSTRAP SAMPLE WITH INDEXES bsperm
    gr = g(bsperm,:);
    Xr = X(bsperm,:);
    % Subtract Wn from Wbs
    if (bsi>0)
        gr = [gr; -gn];
        Xr = [Xr; Xn];
    end
    % Compress data with identical X's
    [Xr, Ind] = sortrows(Xr);
    gr = gr(Ind);
    Xu = unique(Xr,'rows');
    nu  = size(Xu,1); % number of unique covariate vectors
    gu = zeros(nu,1);
    jj = 1;
    for j = 1:size(Xr,1)
        if ~(sum(Xr(j,:)~=Xu(jj,:)))
            gu(jj) = gu(jj) + gr(j);
        else
            jj = jj+1;
            gu(jj) = gu(jj) + gr(j);
        end
    end

    % Add explicit monotonicity constraints
    % Xu is ordered by increasing education (Xu(:,2))
    % and then by increasing income (Xu(:,3))
    % For each level of education we impose treatment set inclusion
    sameedu = (Xu(1:end-1,2)==Xu(2:end,2));
    % decreasing in pre-program earnings
    Mineq_d = [diag(sameedu) zeros(nu-1,1)] + [zeros(nu-1,1) diag(-sameedu)];
    % increasing in pre-program earnings
    Mineq_i = [diag(-sameedu) zeros(nu-1,1)] + [zeros(nu-1,1) diag(sameedu)];

    f = [zeros(2*k,1); zeros(2*nu,1); -gu]; % objective function coefficients
    f_n = [zeros(2*k,1); zeros(2*nu,1); gu]; % Reverse objective function coefficients
    B = 1; % bounds on coefficients
    C = B*sum(abs(Xu),2); % maximum values of x'beta
    minmargin = max(1,C)*(1e-8); % prevent non-integer numbers the integrality constraint of integers from being counted as integers

    % for quadrand rules - selected X's
    Xu1 = Xu(:,[1 2]);
    Xu2 = Xu(:,[1 3]);
    % padding:
    ZX = zeros(size(Xu1));
    ZC = zeros(size(diag(C)));
    ZI = eye(nu);
    % AND operation
    % for z_j:  -X'beta + Czj < C,  X'beta - Czj <= 0 
    % for z_*:  z1 + z2 - z* <= 1+eps,  -z1 - z2 + 2z* <= eps
    Aineq_d = [[-Xu1 ZX  diag(C) ZC ZC]; ... 
             [ Xu1 ZX -diag(C) ZC ZC]; ...
             [ZX -Xu2 ZC  diag(C) ZC]; ...
             [ZX  Xu2 ZC -diag(C) ZC]; ...
             [ZX ZX ZI ZI -ZI]; ...
             [ZX ZX -ZI -ZI 2*ZI]; ...
             [zeros(nu-1,2*k+2*nu) Mineq_d]];
    Aineq_i = [[-Xu1 ZX  diag(C) ZC ZC]; ... 
             [ Xu1 ZX -diag(C) ZC ZC]; ...
             [ZX -Xu2 ZC  diag(C) ZC]; ...
             [ZX  Xu2 ZC -diag(C) ZC]; ...
             [ZX ZX ZI ZI -ZI]; ...
             [ZX ZX -ZI -ZI 2*ZI]; ...
             [zeros(nu-1,2*k+2*nu) Mineq_i]];
    bineq = [[(C -minmargin)]; ...
             [   -minmargin]; ...
             [(C -minmargin)]; ...
             [   -minmargin]; ...
             [(1+minmargin)]; ...
             [minmargin]; ...
             minmargin(1:nu-1,:)];

    lb = [-B*ones(2*k,1); zeros(3*nu,1)];
    ub = [ B*ones(2*k,1);  ones(3*nu,1)];
    
    % Variable type string
    ctype = strcat(repmat('C',1,2*k),repmat('B',1,3*nu));

    % Welfare maximization with treatment decreasing in earnings
    [sol_pd, v_pd] = cplexmilp(f,Aineq_d,bineq,[],[],[],[],[],lb,ub,ctype,[],opt);
    %                      with treatment increasing in earnings
    [sol_pi, v_pi] = cplexmilp(f,Aineq_i,bineq,[],[],[],[],[],lb,ub,ctype,[],opt);
    
    if (bsi==0)
        if (v_pd < v_pi)
            beta = sol_pd(1:2*k,:);
            v    = -v_pd;
        else
            beta = sol_pi(1:2*k,:);
            v    = -v_pi;
        end
    else
        vhats_p(bsi) = -min(v_pd,v_pi);
        % "Welfare minimization" for two-sided bootstrap C.I.
        [sol_nd, v_nd] = cplexmilp(f_n,Aineq_d,bineq,[],[],[],[],[],lb,ub,ctype,[],opt);
        [sol_ni, v_ni] = cplexmilp(f_n,Aineq_i,bineq,[],[],[],[],[],lb,ub,ctype,[],opt);
        vhats_n(bsi) = -min(v_nd,v_ni);
        
    end
    
    % DRAW BOOTSTRAP INDEXES FOR NEXT SAMPLE
    bsperm = randi(n,[n 1]);
    
    % progress markers
    bsi
    toc
end

% Save coefficients and other variables for graphs
save(filename_coefs,'beta');
save(filename_results);

in_Ghat = ((X(:,[1 2])*beta(1:2)>0).*(X(:,[1 3])*beta(3:4)>0));

% Print results for the comparison table
fprintf('Treatment cost:\n%d\n',tcost);
fprintf('Proportion treated:\n%.2f\n',mean(in_Ghat));
fprintf('Est. welfare gain:\n%.0f\n',mean(g.*in_Ghat)*Yscale);

% two-sided 95% confidence interval for the welfare gain
vhats_abs = max([vhats_p vhats_n],[],2);

fprintf('Lower 95%% CI:\n%.0f\n',(v-prctile(vhats_abs,95))*Yscale/n);
fprintf('Upper 95%% CI:\n%.0f\n',(v+prctile(vhats_abs,95))*Yscale/n);

% visualize the treatment rule
gscatter(edu, prevearn, in_Ghat, 'br', '..');

diary off
