% Replication file for
% "Who Should Be Treated?
%  Welfare Maximization Methods for Treatment Choice"
% by Toru Kitagawa and Aleksey Tetenov
% 2017-11-27
% Compute quadrant EWM treatment rules in (education, prior earnings)
% using grid search
% Runtime (with 399 bootstrap reps):
% ~8 minutes on a 4-core 4GHz Intel i7-6700K processor

clear all
rng(0); % Starting point for the RNG
bsimax = 399; % number of bootstrap reps, 0 = no bootstrap
cost = false; % switch: take treatment costs into account or not
if (cost)
    tcost = 774;
    diary('jtpa_quadb_cost.log');
    filename_coefs = 'jtpa_quadb_cost_coefs.mat';
    filename_results = 'jtpa_quadb_cost_results.mat';
else
    tcost = 0;
    diary('jtpa_quadb.log');
    filename_coefs = 'jtpa_quadb_coefs.mat';
    filename_results = 'jtpa_quadb_results.mat';
end

% DATA INPUT AND PREPARATION
load('../jtpa_kt.mat')

Y = earnings-tcost.*D;
meanY = mean(Y);
Y = Y - meanY; % demean outcomes
n = length(Y); % sample size
ps = 2/3; % propensity score
X  = [edu prevearn];

g = Y.*(D/ps - (1-D)/(1-ps)); % elements of W(G) - W(G_0)

% Compress data with identical X's
[Xr, Ind] = sortrows(X);
gr = g(Ind);
Xu = unique(Xr,'rows');
nu  = size(Xu,1); % number of unique covariate vectors
gu = zeros(nu,1);
nw = zeros(nu,1);
jj = 1;
for j = 1:size(Xr,1)
    if ~(sum(Xr(j,:)~=Xu(jj,:)))
        gu(jj) = gu(jj) + gr(j);
        nw(jj) = nw(jj) + 1;
    else
        jj = jj+1;
        gu(jj) = gu(jj) + gr(j);
        nw(jj) = nw(jj) + 1;
    end
end
x1 = Xu(:,1);
x2 = Xu(:,2);

% Create grids of midpoints between distinct values of x1/x2
u_x1 = unique(x1); eps_x1 = min(u_x1(2:end) - u_x1(1:end-1))/2;
grid_x1 = [u_x1(1)-eps_x1; (u_x1(2:end) + u_x1(1:end-1))/2; u_x1(end)+eps_x1];
k1 = size(grid_x1,1);
u_x2 = unique(x2); eps_x2 = min(u_x2(2:end) - u_x2(1:end-1))/2;
grid_x2 = [u_x2(1)-eps_x2; (u_x2(2:end) + u_x2(1:end-1))/2; u_x2(end)+eps_x2];
k2 = size(grid_x2,1);

% Maximize the EW criterion
maxW = -Inf;
for i1=1:k1
    for i2=1:k2
        for sign1 = -1:2:1
            for sign2 = -1:2:1
                select = (sign1*(x1 - grid_x1(i1))>=0).*(sign2*(x2 - grid_x2(i2))>=0);
                W = sum(gu.*select);
                if (W > maxW)
                    max_i1 = i1;
                    max_i2 = i2;
                    max_sign1 = sign1;
                    max_sign2 = sign2;
                    maxW = W;        
                end
            end
        end
    end
end

vhats_p = zeros(bsimax,1); 
vhats_n = zeros(bsimax,1); 

for bsi = 1:bsimax
    % DRAW BOOTSTRAP INDEXES
    bsperm = randi(n,[n 1]);
    % Subtract Wn from Wbs
    gr = [g(bsperm,:); -g];
    Xr = [X(bsperm,:); X];
    % Compress the data with identical X's
    [Xr, Ind] = sortrows(Xr);
    gr = gr(Ind);
    bXu = unique(Xr,'rows');
    bnu  = size(Xu,1); % number of unique covariate vectors
    bgu = zeros(nu,1);
    jj = 1;
    for j = 1:size(Xr,1)
        if ~(sum(Xr(j,:)~=bXu(jj,:)))
            bgu(jj) = bgu(jj) + gr(j);
        else
            jj = jj+1;
            bgu(jj) = bgu(jj) + gr(j);
        end
    end
    % Maximize and minimize Wn-Wbs
    maxWb = -Inf; minWb = Inf;
    for i1=1:k1
        for i2=1:k2
            for sign1 = -1:2:1
                for sign2 = -1:2:1
                    select = (sign1*(x1 - grid_x1(i1))>=0).*(sign2*(x2 - grid_x2(i2))>=0);
                    Wb = sum(bgu.*select);
                    maxWb = max(maxWb,Wb);
                    minWb = min(minWb,Wb);
                end
            end
        end
    end
    vhats_p(bsi) = maxWb;
    vhats_n(bsi) = minWb;
end

% Save coefficients and other variables for graphs
save(filename_coefs,'max_i1','max_i2','max_sign1','max_sign2','maxW');
save(filename_results);

in_Ghat = (max_sign1*(x1 - grid_x1(max_i1))>=0).*(max_sign2*(x2 - grid_x2(max_i2))>=0);

% Print results for the comparison table
fprintf('Treatment cost:\n%d\n',tcost);
fprintf('Proportion treated:\n%.2f\n',sum(nw.*in_Ghat)/n);
fprintf('Est. welfare gain:\n%.0f\n',sum(gu.*in_Ghat)/n);

% two-sided 95% confidence interval for the welfare gain
vhats_abs = max([vhats_p -vhats_n],[],2);

fprintf('Lower 95%% CI:\n%.0f\n',(maxW-prctile(vhats_abs,95))/n);
fprintf('Upper 95%% CI:\n%.0f\n',(maxW+prctile(vhats_abs,95))/n);

% visualize the treatment rule
gscatter(Xu(:,1), Xu(:,2), in_Ghat, 'br', '..');

diary off
